"""Ошибки"""


class ReqFieldMissingError(Exception):
    """Исключение - отсутствует обязательное поле в принятом словаре"""

    def __init__(self, missing_field):
        self.missing_field = missing_field

    def __str__(self):
        return f'В принятом словаре отсутствует обязательное поле {self.missing_field}.'


class IncorrectDataRecivedError(Exception):
    """Исключение  - некорректные данные получены от сокета """

    def __str__(self):
        return 'Принято некорректное сообщение от удалённого компьютера.'


class ServerError(Exception):
    """Исключение - ошибка сервера"""

    def __init__(self, text):
        self.text = text

    def __str__(self):
        return self.text



"""
конфигурация серверного логгера
"""

import logging
from logging import handlers as handlers

# Создаём объект-логгер с именем app.server
import os

SERVER_LOG = logging.getLogger('app.server')

# Создаём объект форматирования:
SERVER_FORMATTER = logging.Formatter("%(asctime)s %(levelname)s %(filename)s %(message)s ")

# Создаём файловый обработчик логирования:
PATH = os.path.dirname(os.path.abspath(__file__))
PATH = os.path.join(PATH, 'app.server.log')
FILE_HANDLER = handlers.TimedRotatingFileHandler(PATH, encoding='utf-8', when="D", interval=1)
# Задаем форматтер для обработчика
FILE_HANDLER.setFormatter(SERVER_FORMATTER)

# Добавляем в логгер обработчик  и задаем уровень логгирования
SERVER_LOG.addHandler(FILE_HANDLER)
SERVER_LOG.setLevel(logging.DEBUG)

if __name__ == '__main__':
    SERVER_LOG.debug('Отладочное сообщение от конф. файла')



import sys
import os
import unittest
import json

sys.path.insert(0, os.path.join(os.getcwd(), '..'))

from common.variables import RESPONSE, ERROR, USER, ACCOUNT_NAME, TIME, ACTION, PRESENCE, ENCODING
from common.utils import get_message, send_message


class TestSocket:
    """Тестовый класс для имитации сокета - для проверки корректности методов send() и receive(),
    при создании необходимо задать сообщение(словарь)
    """

    def __init__(self, test_message):
        self.test_message = test_message
        self.encoded_message = None
        self.sent_message = None

    def send(self, sent_message):
        """Тестовый метод отправки, он корректно кодирует сообщение (self.encoded_message),
        а так-же сохраняет результат  работы тестируемой функции send() (message_to_send ->self.sent_message) .
        """
        json_test_message = json.dumps(self.test_message)
        # кодируем сообщение
        self.encoded_message = json_test_message.encode(ENCODING)
        # здесь сохраняем то, что отправлено в сокет тестируемой функцией
        self.sent_message = sent_message

    def recv(self, max_len):
        """Получаем данные из сокета"""
        json_test_message = json.dumps(self.test_message)
        return json_test_message.encode(ENCODING)


class TestSocket_resv_no_coding:
    """Тестовый класс для имитации сокета - для проверки корректности методов send() и receive(),
    при создании необходимо задать сообщение(словарь)
    """

    def __init__(self, test_message):
        self.test_message = test_message

    def recv(self, max_len):
        """Получаем данные из сокета без кодирования!"""
        json_test_message = json.dumps(self.test_message)
        return json_test_message


class TestUtils(unittest.TestCase):
    """класс юнит-тестов функций модуля утилит (common/utils) """
    test_message_to_send = {
        ACTION: PRESENCE,
        TIME: 20.00,
        USER: {
            ACCOUNT_NAME: 'user'
        }
    }
    test_msg_receive_ok = {
        RESPONSE: 200
    }
    test_msg_receive_err = {
        RESPONSE: 400,
        ERROR: 'Bad Request'
    }
    test_msg_receive_non_dict = 'non_dict_message'

    def test_send_message_ok(self):
        """Тест функции отправки сообщения.
        Создадим тестовый сокет и проверим корректность отправки сообщения
        """
        # создаемэкземпляр тестового сокета
        test_socket = TestSocket(self.test_message_to_send)
        # вызываем тестируемую функцию, результаты сохраняем в тестовом сокете (sent_message)
        send_message(test_socket, self.test_message_to_send)
        # Проверяем корректности кодирования сообщения тестовой функцией
        # (сравниваем результат кодирования функцией send() с тестовым кодированием в иммитации сокета)
        self.assertEqual(test_socket.sent_message, test_socket.encoded_message)

    def test_send_message_error(self):
        """Тест функции отправки - обработка исключения неверного формата сообщения.
        Создаем тестовый сокет и проверяем корректность обработки неверного формата сообщения (TypeError)
        """
        # создаем экземпляр тестового сокета
        test_socket = TestSocket(self.test_message_to_send)
        self.assertRaises(TypeError, send_message, test_socket, "non_dict_message")

    def test_get_message_ok(self):
        """Тест функции приёма сообщения о корректной доставке сообщения."""
        test_sock_ok = TestSocket(self.test_msg_receive_ok)
        # тест корректной расшифровки корректного словаря
        self.assertEqual(get_message(test_sock_ok), self.test_msg_receive_ok)

    def test_get_message_error(self):
        """Тест функции приёма сообщения об ошибке"""
        test_sock_err = TestSocket(self.test_msg_receive_err)
        # тест корректной обработки сообщения об ошибке
        self.assertEqual(get_message(test_sock_err), self.test_msg_receive_err)

    def test_get_message_value_error(self):
        """Тест функции получения - обработка исключения неверного формата сообщения (не словарь).
        Создаем тестовый сокет и проверяем корректность обработки неверного формата сообщения (ValueError)
        """
        # создаем экземпляр тестового сокета
        test_sock_non_dict_msg = TestSocket(self.test_msg_receive_non_dict)
        self.assertRaises(ValueError, get_message, test_sock_non_dict_msg)

    def test_get_message_value_error_no_coding(self):
        """Тест функции получения - обработка исключения неверного формата вх.сообщения (не byte).
        Создаем тестовый сокет (для теста ошиочнго формата) и проверяем корректность обработки неверного формата сообщения (ValueError)
        """
        # создаем экземпляр тестового сокета
        test_sock_non_coding_msg = TestSocket_resv_no_coding(self.test_msg_receive_ok)
        self.assertRaises(ValueError, get_message, test_sock_non_coding_msg)


if __name__ == '__main__':
    unittest.main()



import sys
import logging

SERVER_LOG = logging.getLogger('app.server')


class PortDescriptor:

    def __set__(self, instance, value):
        try:
            value = int(value)
        except ValueError:
            SERVER_LOG.error(f"Не верный  тип значения порта: {value}")
            sys.exit(1)
        if value < 1024 or value > 65535:
            SERVER_LOG.error(f"Значение порта {value} Не находится в диапазоне 1025 - 65535.")
            sys.exit(1)
        # Если ошибки нет - добавляем порт к атрибутам экземпляра класса
        instance.__dict__[self.my_attr] = value

    def __set_name__(self, owner, my_attr):
        self.my_attr = my_attr



import sys
import logging
import traceback
import socket


def func_to_log(func):
    """Декоратор, выполняющий логирование вызовов функций.
    Сохраняет события типа debug, содержащие
    информацию о имени вызываемой функиции, параметры с которыми
    вызывается функция, и модуль, вызывающий функцию.
    """
    if 'server.py' in sys.argv[0]:
        current_log = logging.getLogger('app.server')
    else:
        current_log = logging.getLogger('app.client')

    def log_writer(*args, **kwargs):
        """Обертка"""
        current_log.debug(
            f'Вызываем функцию {func.__name__} c параметрами {args}, {kwargs}'
            f'из функции {traceback.format_stack()[0].strip().split()[-1]} модуля {func.__module__}.'
        )
        res = func(*args, **kwargs)

        return res

    return log_writer


def login_required(func):
    """Декоратор, проверяющий, что клиент авторизован на сервере.
    Проверяет, что передаваемый объект сокета находится в
    списке авторизованных клиентов.
    За исключением передачи словаря-запроса
    на авторизацию. Если клиент не авторизован,
    генерирует исключение TypeError
    """

    def checker(*args, **kwargs):
        # проверяем, что первый аргумент - экземпляр MessageProcessor
        # Импортить необходимо тут, иначе ошибка рекурсивного импорта.
        # ----------------------------------------------------------------
        # args = (
        #         <MessageProcessor(Thread-5, started daemon 140650633856768)>,
        #         {'action': 'presence',
        #          'time': 1654900198.8001323,
        #           'user': {
        #                    'account_name': 'test1',
        #                    'pubkey': '-----BEGIN PUBLIC KEY-----\nMIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEAmKN/CFSxgU8eu0oO0oKw\n29WTZQSfqw/mJgEtr8nLUAOHGcg3kT7epUgPfwbo/V67sJlGhb/UD7dPK81utWRn\nFKhhUGuo+ad/4HnvbSQjIHy2Wbr85T4gJCL1IqTkodAgSOo4Nuv/Qq9r5po0dNIC\nF4YTZrzfCy6V0v349iXM2CXf+/14fHCxsm3OkNCUwHsOW6nzh5fIyAs1UhssJm/Z\nbCNzX5PkRjI7bwBJhoXHNgS1fDyII6vGrQAyAwxU0hKrBAtAzYIon5ZlIYxyF2/5\nKb8IVmLmnrvCpmtjTQ4u80Dp6YuErMCD82GzgUi50UdQoW617AmFxaKpvO7Lu+aA\nfwIDAQAB\n-----END PUBLIC KEY-----'
        #                    }
        #          },
        #          <socket.socket fd=25, family=AddressFamily.AF_INET, type=SocketKind.SOCK_STREAM, proto=0, laddr=('127.0.0.1', 7777), raddr=('127.0.0.1', 52416)>
        # )
        from server.core import Server
        from common.variables import ACTION, PRESENCE
        if isinstance(args[0], Server):
            found = False
            for arg in args:
                if isinstance(arg, socket.socket):
                    # Проверяем, что данный сокет есть в списке names класса
                    # Server
                    for client in args[0].names:
                        # args[0].names = {'test1': <socket.socket fd=25, family=AddressFamily.AF_INET, type=SocketKind.SOCK_STREAM, proto=0, laddr=('127.0.0.1', 7777), raddr=('127.0.0.1', 52420)>}
                        if args[0].names[client] == arg:
                            found = True

            # Теперь надо проверить, что передаваемые аргументы не presence
            # сообщение. Если presence, то разрешаем
            for arg in args:
                if isinstance(arg, dict):
                    if ACTION in arg and arg[ACTION] == PRESENCE:
                        found = True
            # Если не не авторизован и не сообщение начала авторизации, то
            # вызываем исключение.
            if not found:
                raise TypeError
        return func(*args, **kwargs)

    return checker



DEFAULT_PORT = 7777
DEFAULT_IP_ADDRESS = '127.0.0.1'
MAX_CONNECTIONS = 10
MAX_PACKAGE_LENGTH = 1024
ENCODING = 'utf-8'
# Протокол JIM, осн. ключи
ACTION = 'action'
TIME = 'time'
USER = 'user'
ACCOUNT_NAME = 'account_name'
# Доп. ключи
PRESENCE = 'presence'
RESPONSE = 'response'
ERROR = 'error'
MESSAGE = 'message'
MESSAGE_TEXT = 'mess_text'
SENDER = 'sender'
MESSAGE_RECEIVER = 'message_receiver'
EXIT = 'exit'
GET_CONTACTS = 'get_contacts'
LIST_INFO = 'data_list'
REMOVE_CONTACT = 'remove'
ADD_CONTACT = 'add'
USERS_REQUEST = 'get_users'
DATA = 'bin'
PUBLIC_KEY = 'pubkey'
# Словари - ответы:
# 200
RESPONSE_200 = {RESPONSE: 200}
# 202
RESPONSE_202 = {RESPONSE: 202,
                LIST_INFO:None
                }
# 400
RESPONSE_400 = {
            RESPONSE: 400,
            ERROR: None
        }
# 205
RESPONSE_205 = {
    RESPONSE: 205
}

# 511
RESPONSE_511 = {
    RESPONSE: 511,
    DATA: None
}


SERVER_DB = 'sqlite:///server_db.db3'



from setuptools import setup, find_packages

setup(name="MsgJimPyServer",
      version="1.2.0",
      description="Server part of jim based messenger ",
      author="Roman Lopatin",
      author_email="romanlopatin@gmail.com",
      packages=find_packages(),
      # package_dir={"": "server"},
      include_package_data=True,
      package_data={"": ["*.ini"]},
      install_requires=['PyQt5', 'sqlalchemy'],
      # package_data={"server": ["*.txt", "*.rst", "*.ini"]}
      # scripts=['server/server_run']
      )



import argparse
import sys
import os
import configparser
import threading
import logging

import PyQt5
from PyQt5.QtWidgets import QApplication, QMessageBox

from common.proj_decorators import func_to_log
from server.server_db import ServerStorage
from server.core import Server
from server.main_window import MainWindow
import server.logs.server_log_config

SERVER_LOG = logging.getLogger('app.server')

new_connection = False
conflag_lock = threading.Lock()


@func_to_log
def serv_arg_parser(default_port, default_address):
    """Парсер аргументов коммандной строки."""
    SERVER_LOG.debug(
        f'Инициализация парсера аргументов коммандной строки: {sys.argv}')
    parser = argparse.ArgumentParser()
    parser.add_argument('-p', default=default_port, type=int, nargs='?')
    parser.add_argument('-a', default=default_address, nargs='?')
    parser.add_argument('--no_gui', action='store_true')
    namespace = parser.parse_args(sys.argv[1:])
    listen_address = namespace.a
    listen_port = namespace.p
    gui_flag = namespace.no_gui
    SERVER_LOG.debug('Аргументы успешно загружены.')
    return listen_address, listen_port, gui_flag


def print_help():
    print('Поддерживаемые комманды:')
    print('users - список известных пользователей')
    print('connected - список подключённых пользователей')
    print('loghist - история входов пользователя')
    print('exit - завершение работы сервера.')
    print('help - вывод справки по поддерживаемым командам')


def main():
    config = configparser.ConfigParser()
    ini_path = os.path.join(os.getcwd(), 'server.ini')
    config.read(ini_path)

    database = ServerStorage(
        os.path.join(
            config['SETTINGS']['Database_path'],
            config['SETTINGS']['Database_file'])
    )

    address_to_listen, port_to_listen, gui_flag = serv_arg_parser(
        config['SETTINGS']['Default_port'], config['SETTINGS']['Listen_Address'])
    server_ = Server(address_to_listen, port_to_listen, database)
    server_.daemon = True
    server_.start()

    # Без добавления пути не находит плагин для запуска QApplication!
    dir_name = os.path.dirname(PyQt5.__file__)
    plugin_path = os.path.join(dir_name, 'Qt5', 'plugins', 'platforms')
    os.environ['QT_QPA_PLATFORM_PLUGIN_PATH'] = plugin_path
    # **********************************************

    server_app = QApplication(sys.argv)
    main_window = MainWindow(database, server_, config)

    # Запускаем GUI
    server_app.exec_()


if __name__ == '__main__':
    main()



import json
from common.variables import MAX_PACKAGE_LENGTH, ENCODING
from common.errors import IncorrectDataRecivedError
from common.proj_decorators import func_to_log


@func_to_log
def get_message(sock):
    encoded_msg = sock.recv(MAX_PACKAGE_LENGTH)
    if isinstance(encoded_msg, bytes):
        json_response = encoded_msg.decode(ENCODING)
        response = json.loads(json_response)
        if isinstance(response, dict):
            return response
        else:
            raise IncorrectDataRecivedError
    else:
        raise IncorrectDataRecivedError


@func_to_log
def send_message(sock, message):
    if not isinstance(message, dict):
        raise TypeError
    json_message = json.dumps(message)
    encoded_message = json_message.encode(ENCODING)
    sock.send(encoded_message)



import dis


class ServerVerifier(type):
    """Метакласс ServerVerifier, выполняет базовую проверку класса «Server»:
    отсутствие вызовов connect для сокетов;
    использование сокетов для работы по TCP.
    """

    # Вызывается для создания экземпляра класса, перед вызовом __init__
    def __init__(cls, future_class_name, future_class_parents, clsdict):
        # Список методов, которые используются в функциях класса:
        methods = []  # получаем с помощью 'LOAD_GLOBAL'
        # Обычно методы, обёрнутые декораторами попадают
        # не в 'LOAD_GLOBAL', а в 'LOAD_METHOD'
        methods_2 = []  # получаем с помощью 'LOAD_METHOD'
        # Атрибуты, используемые в функциях классов
        attrs = []  # получаем с помощью 'LOAD_ATTR'
        for func in clsdict:
            try:
                ret = dis.get_instructions(clsdict[func])
            except TypeError:
                pass
            else:
                for i in ret:
                    print(i)
                    # opname - имя для операции
                    if i.opname == 'LOAD_GLOBAL':
                        if i.argval not in methods:
                            # заполняем список методами, использующимися в функциях класса
                            methods.append(i.argval)
                    elif i.opname == 'LOAD_METHOD':
                        if i.argval not in methods_2:
                            # заполняем список методами, использующимися в функциях класса, обернутые декораторами
                            methods_2.append(i.argval)
                    elif i.opname == 'LOAD_ATTR':
                        if i.argval not in attrs:
                            # заполняем список атрибутами, использующимися в функциях класса
                            attrs.append(i.argval)
        # Если обнаружено использование недопустимого метода connect, вызываем исключение:
        if 'connect' in methods:
            raise TypeError('Использование метода connect недопустимо в серверном классе')
        # Если сокет не инициализировался константами SOCK_STREAM(TCP) AF_INET(IPv4), тоже исключение.
        if not ('SOCK_STREAM' in attrs and 'AF_INET' in attrs):
            raise TypeError('Некорректная инициализация сокета.')

        super(ServerVerifier, cls).__init__(future_class_name, future_class_parents,
                                            clsdict)


# Метакласс для проверки корректности клиентов:
class ClientVerifier(type):
    """Метакласс ClientVerifier, выполняет базовую проверку класса «Client»
    (для некоторых проверок уместно использовать модуль dis):
    отсутствие вызовов accept и listen для сокетов;
    использование сокетов для работы по TCP;
    """

    def __init__(cls, future_class_name, future_class_parents, clsdict):
        # Список методов, которые используются в функциях класса:
        methods = []
        for func in clsdict:
            # Пробуем
            try:
                ret = dis.get_instructions(clsdict[func])
                # Если не функция то ловим исключение
            except TypeError:
                pass
            else:
                # Если функция разбираем код, получая используемые методы.
                for i in ret:
                    if i.opname == 'LOAD_GLOBAL':
                        if i.argval not in methods:
                            methods.append(i.argval)
        # Если обнаружено использование недопустимого метода accept, listen, socket бросаем исключение:
        # pprint(methods)
        for command in ('accept', 'listen'):
            if command in methods:
                raise TypeError('В классе обнаружено использование запрещённого метода')
        # Вызов get_message или send_message из utils считаем корректным использованием сокетов
        if 'get_message' in methods or 'send_message' in methods or True:
            pass
        else:
            raise TypeError('Отсутствуют вызовы функций, работающих с сокетами.')
        super().__init__(future_class_name, future_class_parents, clsdict)
